//************************************************************************
// Bitmap.C
//
// Bitmaps an beliebige Position zeichnen.
// Der Nullpunkt der Anzeige ist links oben.
//
// Zum erzeugen einer Bitmap benutze mein "bmp2c".
// Aufruf: "bmp2c paul.bmp v"
// Parameter "v" erzeugt fr KS108 optimierten Bitmap-C-Code.
//
// Routine wurde optimiert fr den Fall das ypos der Bitmap auf den Adressen
// 0,8,16,24,32,40,48,56 beginnt. Dann kann man statt nur einzelne
// Pixel zu setzen gleich ein ganzes Byte mit 8 Pixeln schreiben.
// Bitmaps werden dann mehr als 8 mal schneller angezeigt !
//
// hk@holger-klabunde.de
// http://www.holger-klabunde.de
//************************************************************************
// 14.12.2007
//************************************************************************
// Compiler WinAVR 4.1.2
//************************************************************************
#include <avr/pgmspace.h>

#include "bitmap.h"
#include "ks108.h"

//##########################################################
// Loads a bitmap from FLASH 
void LoadBitmap_P(unsigned const char *bitmap,unsigned char xpos , unsigned char ypos, unsigned char width, unsigned char height)
//##########################################################
{
 unsigned char i,j,k,yp,xp,bheight;
 unsigned char by,mask;
 unsigned int xend;     //xpos + width kann grer als 255 werden

 xend=(unsigned int)xpos + (unsigned int)width;
 if(xend>LCD_WIDTH) xend=LCD_WIDTH; //Bitmap abschneiden wenn auerhalb des Displays
 
 bheight=height/8; //Anzahl Bytes vertikal
 if((height%8)!=0) bheight++; //Bei Rest noch ein Byte mehr

 yp=ypos;
 for(i=0; i<bheight; i++) //ber die gesamte Hhe in Bytes
  {
   xp=xpos; //Zum Beginn der Zeile

   for(j=0; j<width; j++) //ber die gesamte Breite in Pixeln
    {
     by=pgm_read_byte(bitmap);

     if( (yp % 8) != 0) // Das heisst langsam zeichnen, Pixel fr Pixel
      {
       mask=0x01;         //Fngt bei LSB an

       for(k=0; k<8; k++) //ber ein Byte
        {
          //Nur auf dem Display vorhandene Pixel zeichnen
         if(xp < (unsigned char)xend)
          {
           SetPixel(xp,yp+k,(by & mask) );
          }                               

         mask<<=1; //Nchstes Bit
        }//for k

      }//if( (yp % 8) != 0)
     else // Schnell zeichnen, 8 Pixel auf einmal
      {

       //Nur auf dem Display vorhandene Pixel zeichnen
       if(xp < (unsigned char)xend)
          {
           if(SetPosition(xp,yp)) //Chip und Adresse einstellen
            {
             WriteDisplay(by);   //8 Pixel schreiben
            } 
          }
      }//if( (yp % 8) != 0) 

     xp++; 
     bitmap++;
    } //for j

   yp+=8; //Nchste Zeile
  } //for i
}

