//************************************************************************
// KS108.C
// LowLevel Routinen fr LCD-Displays mit KS0108
// mit bis zu vier KS108 Chips also bis 240x64
//
// Der Nullpunkt des Displays liegt oben,links.
//
// hk@holger-klabunde.de
// http://www.holger-klabunde.de
//************************************************************************
// 14.12.2007
//************************************************************************
// Compiler WinAVR 4.1.2
//************************************************************************
#include <avr/io.h>

#include "ks108.h" // Das muss vor <util/delay.h> stehen !

//Note: 1. F_CPU im makefile definieren, 2. Optimization ein
#include <util/delay.h>

//###################################################################################
// E ist die Bremse des Displays. Braucht ganz schn lange Delays.
// Geniale Angaben zum Timing im Datenblatt:
// 450ns Minimum fr E_low und E_high. Zusammen 1000ns. Fehlen schonmal 100ns.
// Und wo ist das Maximum ? Da ist raten angesagt.
void Wait_for_E(void)
//###################################################################################
{
 _delay_us(KS108_E_DELAY);
}

//##########################################################
// Liest ein Byte aus dem Display
unsigned char ReadDisplay(void)
{
 unsigned char by;

 CheckBusy();

 RS_ON;
 RW_ON(); //Wir wollen lesen

//Display mu zweimal gelesen werden !
//Dummy Read
 E_ON();
 E_OFF();

 CheckBusy();
 DATA_DIR_IN();

 RS_ON;
 RW_ON(); //Wir wollen lesen

 E_ON();
 by=READ_DATA; //DATA RAM lesen
 E_OFF();

 RW_OFF(); //zurck auf schreiben

 DATA_DIR_OUT;
 
 return by;
}

//##########################################################
//Ein Byte in das Display schreiben
//Register Select mu vorher entsprechend gesetzt werden !
void WriteDisplay(unsigned char dat)
{
 DATA_DIR_OUT;

 RW_OFF(); //Display schreiben setzen

 WRITE_DATA(dat);
 E_ON();
 E_OFF();

}

//##########################################################
//Pixel auf dem Display setzen
//mode=0; Pixel lschen
//mode=1; Pixel setzen
unsigned char SetPixel(unsigned char xpos, unsigned char ypos, unsigned char mode)
{
 unsigned char by,tmp,pixpos;
 unsigned char pixset;

 if(xpos>=LCD_WIDTH) return 1; //Punkt auerhalb der Anzeige
 if(ypos>=LCD_HEIGHT) return 1;

 pixset=mode; //Pixel gesetzt oder nicht erstmal merken

 SetPosition(xpos,ypos); //Position auf 8 Pixel genau setzen

 by=ReadDisplay();       //8 Pixel lesen

 tmp= ypos % 8; //Position des Pixels im Byte
 pixpos= 1 << tmp;             //An die richtige Position schieben

/*
 if(pixelmode==P_REPLACE) //Pixel ersetzen
  { //Pixel wird unten ersetzt
  }
 
 if(pixelmode==P_OR) //Pixel ODER verknpfen, 0-Pixel drfen gesetzt werden
  {
   if(by&pixpos) pixset=1; //Pixel war gesetzt dann bleibt das auch so
   //Wenn pixset=1 wird der Pixel immer gesetzt
  }

 if(drawmode==P_INVERT) //Pixel negativ zeichnen
  {
   if(pixset==0) pixset=1; //invertieren
   else pixset=0;
  }
*/

 if(pixset==1 && by&pixpos) return 0; //Pixel hatte den Wert schon
 if(pixset==0 && (by&pixpos)==0) return 0; //Pixel hatte den Wert schon

 if(pixset==0) by&=~pixpos; //Pixel lschen
 else by|=pixpos; //Pixel setzen

 SetPosition(xpos,ypos); //Position wird nach dem lesen verndert !
                         //Also neu setzen
 WriteDisplay(by);       //8 Pixel schreiben
 return 0;
}

//##########################################################
//Zustand des Pixels an Stelle x,y lesen
unsigned char GetPixel(unsigned char xpos, unsigned char ypos)
{
 int pixel;
 unsigned char by,tmp,pix;

 if(xpos>=LCD_WIDTH) return 1; //Punkt auerhalb der Anzeige
 if(ypos>=LCD_HEIGHT) return 1;

 SetPosition(xpos,ypos); //Position auf 8 Pixel genau setzen

 by=ReadDisplay();       //8 Pixel lesen

 tmp= ypos % 8; //Position des Pixels im Byte
 pix= 1 << tmp;                //An die richtige Position schieben

 if(by&pix) pixel=1; //war gesetzt
 else pixel=0;       //war nicht gesetzt

 return pixel;
}

//##########################################################
//Aus der X-Position die Chipnummer berechnen
unsigned char GetChipNumber(unsigned char xpos)
{
 return xpos/64; //Jeder Chip kontrolliert 64x64 Pixel
}

//##########################################################
//Chipnummer des Displays setzen
void SetChipNumber(unsigned char num)
{
 // Fr 240x64 Displays braucht man 4 ChipSelect Leitungen
 // Fr 128x64 braucht man nur 2 ChipSelect Leitungen 

#ifdef LCD_WIDTH_240

 CS1_OFF;
 CS2_OFF;
 CS3_OFF;
 CS4_OFF;
 
 switch(num)
  {
   case 0: CS1_ON; break;
   case 1: CS2_ON; break;
   case 2: CS3_ON; break;
   case 3: CS4_ON; break;
  }

#endif //#ifdef LCD_WIDTH_240

#ifdef LCD_WIDTH_128

 CS1_OFF;
 CS2_OFF;
 
 switch(num)
  {
   case 0: CS1_ON; break;
   case 1: CS2_ON; break;
  }

#endif //#ifdef LCD_WIDTH_128

}

//##########################################################
//Aus der Y-Position die Pagenummer berechnen
unsigned char GetPageNumber(unsigned char ypos)
{
 return ypos/8; //Jede Page ist 8 Zeilen hoch
}

//##########################################################
//Pagenummer des Chips setzen
unsigned char SetPageNumber(unsigned char num)
{
 unsigned char result;

 result=CheckBusy(); //Nachsehen ob Reset und Busy 0 sind
 if(result>0) return result; //Timeout vom Display !! nix tun.

 RS_OFF; //Instruction setzen
 WriteDisplay(0xB8|num); //Pagenummer setzen

 RS_ON; //Zurck auf Daten
 return 0;
}

//##########################################################
//Spaltennummer (X-Position) des Chips setzen
unsigned char SetRow(unsigned char xpos)
{
 unsigned char offs;
 unsigned char result;

 result=CheckBusy(); //Nachsehen ob Reset und Busy 0 sind
 if(result>0) return result; //Timeout vom Display !! nix tun.

 RS_OFF; //Instruction setzen

//Spaltenoffset im Chip berechnen
 offs=xpos % 64; //Rest von xpos durch 64 bestimmen

 WriteDisplay(0x40|offs); //Spaltenummer setzen
 RS_ON; //Zurck auf Daten
 return 0;
}

//##########################################################
//Gewnschte Koordinate auf dem Display einstellen
unsigned char SetPosition(unsigned char xpos, unsigned char ypos)
{
 unsigned char chip,page;

 if(xpos>=LCD_WIDTH) return 0; //Punkt auerhalb der Anzeige
 if(ypos>=LCD_HEIGHT) return 0;

 chip=GetChipNumber(xpos);
 page=GetPageNumber(ypos);

 SetChipNumber(chip); //Erst den Chip auswhlen
 SetPageNumber(page); //Dann die Page setzen
 SetRow(xpos);        //Zum Schlu die X-Position
 return 1;
}

//##########################################################
//An jeden KS0108 mu der DisplayOn Befehl gesendet werden
void DisplayOn(void)
{
 unsigned char result,i;

 DATA_DIR_OUT; // Data from AVR to display

 RSTB_SET_DDR;
 E_SET_DDR;
 E_OFF();
 RS_SET_DDR;
 RW_SET_DDR;

#ifdef USE_BACKLIGHT  // if you have a backlight !
 BACKLIGHT_SET_DDR;
 BACKLIGHT_ON;
#endif

 CS1_SET_DDR;
 CS2_SET_DDR;

#if defined (LCD_WIDTH_240)
 CS3_SET_DDR;
 CS4_SET_DDR;
#endif

 RSTB_ON;

 for(i=0; i<MAX_CONTROLLERS; i++)
  {
   SetChipNumber(i); 
   result=CheckBusy(); //Nachsehen ob Reset und Busy 0 sind
   if(result>0) return; //Timeout vom Display !! nix tun.

   RS_OFF; //Instruction senden setzen
   WriteDisplay(0x3F); //Befehl Display On

// Tip von Olaf Schulz: Display Start Line bei seinen Displays nach Reset scheinbar nicht ok.
// Das Bild ist dann in Y-Richtung verschoben. Dies soll helfen.
   NOP; NOP;
   WriteDisplay(0xC0); //Befehl Display Start Line 0 fr jeden Controller senden
    
   RS_ON; //Zurck auf Daten
  } 

 // Todo:
 // Eventuelles Backlight einschalten
}

//##########################################################
void ClearScreen(void)
//##########################################################
{
 unsigned char i,j,k;

 k=0;
 do
  {
   SetChipNumber(k); 

   for(j=0; j<8; j++)
    {
     SetPageNumber(j);
     CheckBusy();
     RS_ON;
     RW_OFF(); //Display schreiben setzen

     WRITE_DATA(0);
     for(i=0; i<64; i++)
      { E_ON(); Wait_for_E();
        E_OFF(); Wait_for_E();
      }
    }

  k++;
 }while(k<MAX_CONTROLLERS);

}

//##########################################################
//Vor jeder Aktion des Displays nachsehen ob die letzte noch bearbeitet wird
//Testet Busy und Reset gleichzeitig.
//Beim Busy Check mu nur einmal gelesen werden !!
//
//BusyCheck kann man sich sparen. Keine klare Aussage im Datenblatt dazu
//und funktioniert irgendwie auch nicht. Also lassen wirs.
//Dummy Routine zu Timing Zwecken drin lassen !
unsigned char CheckBusy(void)
{
/*
 unsigned char by;
 unsigned long tout;

 DATA_DIR_IN();

 RW_ON(); //Wir wollen lesen
 RS_OFF; //Busy lesen setzen

 E_ON();
 tout=0; //Timeoutzhler zurcksetzen
 do
  {
   by=READ_DATA;
   tout++;
  }while((by&0x80 || by&0x10) && tout<10000); //Warten bis DB7 und DB4 auf 0 sind
 E_OFF();

 RS_ON; //RS immer auf Daten zurcksetzen


 if(tout>=10000)
  {
   return 1;
  }


 DATA_DIR_OUT;
*/

 return 0; //Schei auf das BusyBit
}
